import { NextFunction, Request, Response } from "express";
import prisma from "../../database/dbclient";
import { z } from "zod";
import { leave_requests_status } from "@prisma/client";

//# Create Attendance

const leaveRequestSchema = z.object({
  leave_request_id: z.number().int().optional(),
  employee_id: z.number().int(),
  leave_type_id: z.number(),
  start_date: z.string(),
  end_date: z.string(),
  reason: z.string().optional(),
  status: z.enum(["Pending", "Approved", "Rejected", "Cancelled"]),
  company_id: z.number(),
});

export const createLeaveForEmployee = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const parsed = leaveRequestSchema.safeParse(req.body);
  if (!parsed.success) {
    console.error(
      "POST > Create Company Department > Failed to parse request body\n",
      parsed.error
    );
    return res.status(400).json(parsed.error);
  }

  const {
    employee_id,
    leave_type_id,
    start_date,
    end_date,
    reason,
    status,
    company_id,
  } = parsed.data;

  try {
    if (start_date == end_date) {
      res.status(201).json({
        message: "start_date and end_date can not be same",
      });

      return;
    }

    const createDeparmentForCompany = await prisma.leave_requests.create({
      data: {
        employee_id,
        leave_type_id,
        start_date: new Date(start_date),
        end_date: new Date(end_date),
        reason: reason,
        status: status,
        company_id: company_id,
      },
    });

    res.status(201).json({
      message: "Department creation successful.",
      data: createDeparmentForCompany,
    });
  } catch (error) {
    next(error);
  }
};

export const updateLeaveForEmployee = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const parsed = leaveRequestSchema.safeParse(req.body);
  if (!parsed.success) {
    return res.status(400).json(parsed.error);
  }

  const { leave_request_id, status } = parsed.data;

  try {
    const createDeparmentForCompany = await prisma.leave_requests.update({
      where: {
        leave_request_id: leave_request_id ?? 0,
      },
      data: {
        status: status,
      },
    });

    res.status(200).json({
      message: "Leave Request Updated successfully.",
      data: createDeparmentForCompany,
    });
  } catch (error) {
    next(error);
  }
};

export const cancelLeaveForEmployee = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const parsed = leaveRequestSchema.safeParse(req.body);
  if (!parsed.success) {
    return res.status(400).json(parsed.error);
  }

  const { leave_request_id, employee_id } = parsed.data;

  try {
    const createDeparmentForCompany = await prisma.leave_requests.update({
      where: {
        leave_request_id: leave_request_id ?? 0,
      },
      data: {
        status: leave_requests_status.Cancelled,
      },
    });

    res.status(201).json({
      message: "Leave Request Cancelled successfully.",
      data: createDeparmentForCompany,
    });
  } catch (error) {
    next(error);
  }
};

export const GetLeavesForList = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { employee_id } = req.params;

  try {
    const createDeparmentForCompany = await prisma.leave_requests.findMany({
      where: {
        employee_id: parseInt(employee_id),
      },
      include: {
        leave_approval_status: {
          select: {
            status: true,
            approval_date: true,
            comments: true,
          },
        },
        leave_types: {
          select: {
            leave_type_name: true,
          },
        },
      },
    });

    res.status(200).json({
      message: "Leave list",
      data: createDeparmentForCompany,
    });
  } catch (error) {
    next(error);
  }
};

export const GetLeaveForOthers = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { company_id } = req.params;
  const { page = 1, limit = 10 } = req.query; // smaller chunk for quick loading

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);

  const skip = (pageNum - 1) * limitNum;

  try {
    var isAdmin = false;

    //paginated

    const leaves = await prisma.leave_requests.findMany({
      where: {
        company_id: parseInt(company_id),
      },
      include: {
        leave_approval_status: {
          select: {
            status: true,
            approval_date: true,
            comments: true,
          },
        },
        leave_types: {
          select: {
            leave_type_name: true,
          },
        },
        employees: {
          select: {
            image: true,
            user: {
              select: {
                first_name: true,
                middle_name: true,
                last_name: true,
              },
            },
            designations: {
              select: {
                designation_name: true,
                leave_allocation: {
                  select: {
                    effective_date: true,
                    expirey_date: true,
                    max_leave: true,
                  },
                },
              },
            },
            departments: {
              select: {
                dpt_name: true,
              },
            },
          },
        },
      },
      skip: skip,
      take: limitNum,
    });

    const total_rows = await prisma.leave_requests.count({
      where: { company_id: Number.parseInt(company_id) },
    });
    const total_page = Math.ceil(total_rows / limitNum);

    res.status(200).json({
      total_page,
      data_count: leaves.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: leaves,
    });
  } catch (error) {
    next(error);
  }
};

export const dashboardStats = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { employee_id } = req.params;

  try {
    const allocatedLeave = await prisma.employees.findUnique({
      where: {
        employee_id: parseInt(employee_id),
      },
      select: {
        leave_requests: false,
        designations: {
          select: {
            leave_allocation: true,
          },
        },
        leave_approvers: true,
      },
    });

    const total_life_time_leave_grouped = await prisma.leave_requests.groupBy({
      by: ["status"],
      _count: {
        leave_request_id: true,
      },
      where: {
        employee_id: parseInt(employee_id),
      },
    });

    const thisyearTotalleave = await prisma.leave_requests.count({
      where: {
        start_date: {
          gte: new Date(new Date().getFullYear(), 0, 1), // First day of this month
          lte: new Date(),
        },
      },
    });

    const thisMonthTotalleave = await prisma.leave_requests.count({
      where: {
        start_date: {
          gte: new Date(new Date().getFullYear(), new Date().getMonth(), 1), // First day of this month
          lte: new Date(),
        },
      },
    });

    res.status(200).json({
      message: "Leave Dashboard",
      status_count: total_life_time_leave_grouped,
      total_leave_current_month: thisMonthTotalleave,
      total_leave_current_year: thisyearTotalleave,
      user: allocatedLeave,
    });
  } catch (error) {
    next(error);
  }
};
